@@ -30,8 +30,10 @@ public:
     {
 
     }
-    virtual void updateData() { m_parent->slotSetDirty(); }
-    virtual void updateCurrent() {}
+    virtual void playlistItemDataHasChanged() Q_DECL_FINAL
+    {
+        m_parent->slotSetDirty();
+    }
 
 private:
     DynamicPlaylist *m_parent;
@@ -57,7 +59,7 @@ DynamicPlaylist::DynamicPlaylist(const PlaylistList &playlists,
     setName(name);
     setAllowDuplicates(false);
 
-    setSorting(columns() + 1);
+    setSortingEnabled(false);
 
     for(PlaylistList::ConstIterator it = playlists.constBegin(); it != playlists.constEnd(); ++it)
         m_observers.append(new PlaylistDirtyObserver(this, *it));
@@ -189,6 +191,4 @@ void DynamicPlaylist::slotUpdateItems()
         synchronizePlayingItems(m_playlists, true);
 }
 
-#include "dynamicplaylist.moc"
-
 // vim: set et sw=4 tw=0 sta:
