@@ -1,5 +1,6 @@
 /**
  * Copyright (C) 2012 Martin Sandsmark <martin.sandsmark@kde.org>
+ * Copyright (C) 2014 Arnold Dumas <contact@arnolddumas.fr>
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -23,43 +24,87 @@
 #include <QNetworkReply>
 #include <QDomDocument>
 #include <QByteArray>
+#include <QUrl>
+#include <QUrlQuery>
 
-#include <kglobal.h>
 #include <kconfiggroup.h>
-#include <KDebug>
 #include <KSharedConfig>
-#include <KSharedPtr>
+
+#include <memory>
 
 #include "tag.h"
+#include "juk.h"
+#include "juk_debug.h"
 
 Scrobbler::Scrobbler(QObject* parent)
     : QObject(parent)
-    , m_networkAccessManager(0)
+    , m_networkAccessManager(nullptr)
+    , m_wallet(Scrobbler::openKWallet())
 {
-    KConfigGroup config(KGlobal::config(), "Scrobbling");
+    QByteArray sessionKey;
 
-    //TODO: use kwallet
-    QString sessionKey = config.readEntry("SessionKey", "");
+    if (m_wallet) {
+        m_wallet->readEntry("SessionKey", sessionKey);
+    } else {
+        KConfigGroup config(KSharedConfig::openConfig(), "Scrobbling");
+        sessionKey.append(config.readEntry("SessionKey", ""));
+    }
 
     if(sessionKey.isEmpty())
         getAuthToken();
 }
 
-Scrobbler::~Scrobbler()
+bool Scrobbler::isScrobblingEnabled() // static
 {
-}
+    QString username, password;
 
-bool Scrobbler::isScrobblingEnabled()
-{
-    KConfigGroup config(KGlobal::config(), "Scrobbling");
+    // checks without prompting to open the wallet
+    if (Wallet::folderDoesNotExist(Wallet::LocalWallet(), "JuK")) {
+        KConfigGroup config(KSharedConfig::openConfig(), "Scrobbling");
 
-    // TODO: use kwallet
-    QString username = config.readEntry("Username", "");
-    QString password = config.readEntry("Password", "");
+        username = config.readEntry("Username", "");
+        password = config.readEntry("Password", "");
+    } else {
+        auto wallet = Scrobbler::openKWallet();
+        if (wallet) {
+            QMap<QString, QString> scrobblingCredentials;
+            wallet->readMap("Scrobbling", scrobblingCredentials);
+
+            if (scrobblingCredentials.contains("Username") && scrobblingCredentials.contains("Password")) {
+                username = scrobblingCredentials["Username"];
+                password = scrobblingCredentials["Password"];
+            }
+        }
+    }
 
     return (!username.isEmpty() && !password.isEmpty());
 }
 
+std::unique_ptr<KWallet::Wallet> Scrobbler::openKWallet() // static
+{
+    using KWallet::Wallet;
+
+    const QString walletFolderName = "JuK";
+    std::unique_ptr<Wallet> wallet(Wallet::openWallet(
+                Wallet::LocalWallet(),
+                JuK::JuKInstance()->winId()));
+
+    if(wallet) {
+        if(!wallet->hasFolder(walletFolderName) &&
+            !wallet->createFolder(walletFolderName))
+        {
+            return nullptr;
+        }
+
+        if(!wallet->setFolder(walletFolderName))
+        {
+            return nullptr;
+        }
+    }
+
+    return wallet;
+}
+
 QByteArray Scrobbler::md5(QByteArray data)
 {
     return QCryptographicHash::hash(data, QCryptographicHash::Md5)
@@ -85,7 +130,7 @@ void Scrobbler::sign(QMap< QString, QString >& params)
 
 void Scrobbler::getAuthToken(QString username, QString password)
 {
-    kDebug() << "Getting new auth token for user:" << username;
+    qCDebug(JUK_LOG) << "Getting new auth token for user:" << username;
 
     QByteArray authToken = md5((username + md5(password.toUtf8())).toUtf8());
 
@@ -98,10 +143,13 @@ void Scrobbler::getAuthToken(QString username, QString password)
 
     sign(params);
 
+    QUrlQuery urlQuery;
     foreach(QString key, params.keys()) {
-        url.addQueryItem(key, params[key]);
+        urlQuery.addQueryItem(key, params[key]);
     }
 
+    url.setQuery(urlQuery);
+
     if (!m_networkAccessManager)
         m_networkAccessManager = new QNetworkAccessManager(this);
 
@@ -111,11 +159,25 @@ void Scrobbler::getAuthToken(QString username, QString password)
 
 void Scrobbler::getAuthToken()
 {
-    KConfigGroup config(KGlobal::config(), "Scrobbling");
+    QString username, password;
 
-    // TODO: use kwallet
-    QString username = config.readEntry("Username", "");
-    QString password = config.readEntry("Password", "");
+    if (m_wallet) {
+
+        QMap<QString, QString> scrobblingCredentials;
+        m_wallet->readMap("Scrobbling", scrobblingCredentials);
+
+        if (scrobblingCredentials.contains("Username") && scrobblingCredentials.contains("Password")) {
+
+            username = scrobblingCredentials["Username"];
+            password = scrobblingCredentials["Password"];
+        }
+
+    } else {
+
+        KConfigGroup config(KSharedConfig::openConfig(), "Scrobbling");
+        username = config.readEntry("Username", "");
+        password = config.readEntry("Password", "");
+    }
 
     if(username.isEmpty() || password.isEmpty())
         return;
@@ -127,10 +189,10 @@ void Scrobbler::handleAuthenticationReply()
 {
     QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
 
-    kDebug() << "got authentication reply";
+    qCDebug(JUK_LOG) << "got authentication reply";
     if(reply->error() != QNetworkReply::NoError) {
         emit invalidAuth();
-        kWarning() << "Error while getting authentication reply" << reply->errorString();
+        qCWarning(JUK_LOG) << "Error while getting authentication reply" << reply->errorString();
         return;
     }
 
@@ -144,24 +206,37 @@ void Scrobbler::handleAuthenticationReply()
 
     if(sessionKey.isEmpty()) {
         emit invalidAuth();
-        kWarning() << "Unable to get session key" << data;
+        qCWarning(JUK_LOG) << "Unable to get session key" << data;
         return;
     }
 
-    KConfigGroup config(KGlobal::config(), "Scrobbling");
-    config.writeEntry("SessionKey", sessionKey);
+    if (m_wallet) {
+
+        m_wallet->writeEntry("SessionKey", sessionKey.toUtf8());
+
+    } else {
+
+        KConfigGroup config(KSharedConfig::openConfig(), "Scrobbling");
+        config.writeEntry("SessionKey", sessionKey);
+    }
+
     emit validAuth();
 }
 
 void Scrobbler::nowPlaying(const FileHandle& file)
 {
-    KConfigGroup config(KGlobal::config(), "Scrobbling");
+    QString sessionKey;
 
-    //TODO: use kwallet
-    QString sessionKey = config.readEntry("SessionKey", "");
-    if(sessionKey.isEmpty()) {
-        getAuthToken();
-        return;
+    if (m_wallet) {
+
+        QByteArray sessionKeyByteArray;
+        m_wallet->readEntry("SessionKey", sessionKeyByteArray);
+        sessionKey = QString::fromLatin1(sessionKeyByteArray);
+
+    } else {
+
+        KConfigGroup config(KSharedConfig::openConfig(), "Scrobbling");
+        sessionKey = config.readEntry("SessionKey", "");
     }
 
     if (!m_file.isNull()) {
@@ -186,10 +261,19 @@ void Scrobbler::nowPlaying(const FileHandle& file)
 
 void Scrobbler::scrobble()
 {
-    KConfigGroup config(KGlobal::config(), "Scrobbling");
+    QString sessionKey;
 
-    //TODO: use kwallet
-    QString sessionKey = config.readEntry("SessionKey", "");
+    if (m_wallet) {
+
+        QByteArray sessionKeyByteArray;
+        m_wallet->readEntry("SessionKey", sessionKeyByteArray);
+        sessionKey = QString::fromLatin1(sessionKeyByteArray);
+
+    } else {
+
+        KConfigGroup config(KSharedConfig::openConfig(), "Scrobbling");
+        sessionKey = config.readEntry("SessionKey", "");
+    }
 
     if(sessionKey.isEmpty()) {
         getAuthToken();
@@ -203,7 +287,7 @@ void Scrobbler::scrobble()
         return; // API says not to scrobble if the user didn't play long enough
     }
 
-    kDebug() << "Scrobbling" << m_file.tag()->title();
+    qCDebug(JUK_LOG) << "Scrobbling" << m_file.tag()->title();
 
     QMap<QString, QString> params;
     params["method"] = "track.scrobble";
@@ -221,6 +305,10 @@ void Scrobbler::scrobble()
 
 void Scrobbler::post(QMap<QString, QString> &params)
 {
+    if(!m_networkAccessManager) {
+        return;
+    }
+
     QUrl url("http://ws.audioscrobbler.com/2.0/");
 
     QByteArray data;
